import glob
import os
from pathlib import Path
from bs4 import BeautifulSoup
from flask import Flask, request
from grobid_client.grobid_client import GrobidClient


def requestGC() -> None:

    """
    Create the GROBID client

    Args:
        None

    Returns:
        None

    Authors:
        Marie FU
    """
    
    client = GrobidClient(config_path="./config.json")
    client.process(service="processFulltextDocument", input_path="./book", output="./book", segment_sentences=True, consolidate_header=True)
    print("request finished")


def getXMLContent() -> BeautifulSoup:

    """
    Get given file content

    Args:
        None

    Returns:
        (BeautifulSoup) : soup of the xml file

    Authors:
        Marie FU
    """

    tei_doc = './book/file.grobid.tei.xml'
    with open(tei_doc, 'r') as tei:
        return BeautifulSoup(tei, 'xml')
    

def getBookTitle(soup : BeautifulSoup) -> (str | None):
        
    """
    Get the title of the PDF book

    Args:
        soup (BeautifulSoup) : xml soup

    Returns:
        (String) : title of the book

    Authors:
        Marie FU
    """
        
    if soup.title is not None:
        return soup.title.string
    else:
        return None


def getDate(soup : BeautifulSoup) -> str:
        
    """
    Get the publication date of the PDF book

    Args:
        soup (BeautifulSoup) : xml soup

    Returns:
        res (String) : date of publication

    Authors:
        Marie FU
    """
        
    res = ""
    date = soup.find('date')
    if date is not None:
        if date is str:
            try:
                res = date['when']
            except TypeError:
                res = "1\t1\t1"
        else:
            res = "1\t1\t1"
    if "-" in res:
        res = res.replace("-","\t")

    dateTab = res.split("\t")
    if len(dateTab) == 2 :
        res += "\t1"
    elif len(dateTab) == 1 :
        res += "\t1\t1"

    return res


def getAuthors(soup : BeautifulSoup) -> str:
        
    """
    Get the authors of the PDF book

    Args:
        soup (BeautifulSoup) : xml soup

    Returns:
        authors (String) : all authorsof the book

    Authors:
        Marie FU
    """
        
    authors = ""
    authorList = []
    if soup.sourceDesc is not None:
        authorList = soup.sourceDesc.findAll("author")

    for author in authorList:
        if author is not None:
            if authors != "" and (author.forename is not None or author.surname is not None):
                authors += "; "
            forename_list = author.findAll("forename")
            forename_ = ""
            for forename in forename_list:
                if forename is not None:
                    if forename_ != "":
                        forename_ += " "
                    forename_ += forename.string
            authors += forename_
            surname_list = author.findAll("surname")
            surname_ = ""
            for surname in surname_list:
                if surname is not None:
                    if surname_ != "":
                        surname_ += " "
                    surname_ += surname.string
            authors += " " + surname_
            if author.roleName is not None and (author.forename is not None or author.surname is not None):
                authors += ", " + author.roleName.string
    if len(authors) == 0:
        authors = "Unknown authors"
    return authors.strip()


def empty_content() -> None:
    for f in glob.glob("./book/file*"):
        os.remove(f)


def getData(soup : BeautifulSoup, title : str|None, date : str, authors : str) -> str:
        
    """
    Get parsed content of the PDF book

    Args:
        soup (BeautifulSoup) : xml soup
        title (String) : title
        date (String) : date of publication
        authors (String) : all authors

    Returns:
        None

    Authors:
        Marie FU
    """
        
    fileData = ""
    if soup.body is not None:
        divList = soup.body.find_all('div')
        
        # Loop in sections
        for div in divList:
            source = ""
            if div is not None:
                paragraphList = div.findAll('p')

                if div.head is not None:
                    source = div.head.string
                else:
                    source = None

            # Loop in paragraphs
            for paragraph in paragraphList:
                fileData += date + "\t" + authors + "\t"
                if source is not None and title is not None:
                    fileData += title + ", " + source + "\t"
                elif source is None and title is not None:
                    fileData += title + "\t"
                elif source is not None and title is None:
                    fileData += "Unknown Book, " + source + "\t"
                else:
                    fileData += "Unknown Book\t"

                if paragraph is not None:
                    sentenceList = paragraph.findAll('s')

                # First sentence of paragraph checker
                first = True
                # Loop in sentences
                for sentence in sentenceList:
                    if sentence is not None and len(sentence.get_text()) != 0:
                        if first:
                            fileData += sentence.get_text() + "\t" + sentence.get_text()
                            first = False
                        else:
                            fileData += " " + sentence.get_text()
                if first:
                    first = False
                    fileData += "Title\tAbstract"
                fileData += "\n"

    return fileData


def checkXMLFile() -> bool:
    if os.path.exists("./book/file.pdf"):
        return True
    empty_content()
    return False


def checkContent(data:str) -> str:
    res = ""
    for line in data.split("\n"):
        if line != "":
            if not line[0].isnumeric():
                res = res[:-1] + line + "\n"
            else:
                res += line + "\n"
    return res.replace("\"","”")


def create_app():
    app = Flask(__name__)

    @app.route("/getFile", methods=['POST'])
    def getBookTSV() -> str:
        if request.method == 'POST':
            # check if the post request has the file part
            if 'dir' not in request.form:
                return "no directory path, curl needs to be like -- curl -X POST -F dir='my_directory_path' url --\n"
            elif not os.path.exists(request.form["dir"]):
                print(request.form["dir"])
                return "directory does not exist, directory path should be absolute\n"
            else:
                dirName = request.form["dir"]
                files = Path(dirName).glob('*.pdf')
                allData = ""
                for file in files:
                    print(file.name)
                    with open("book/file.pdf","wb") as f:
                        f.write(file.read_bytes())
                    requestGC()

                    if not checkXMLFile():
                        return ""

                    soup = getXMLContent()

                    title = getBookTitle(soup)

                    date = getDate(soup)

                    authors = getAuthors(soup)

                    allData += getData(soup, title, date, authors)

                    empty_content()

                return "Publication Year\tPublication Month\tPublication Day\tAuthors\tSource\tTitle\tAbstract\n" + checkContent(allData)
        return ""

    return app

create_app()
